package edu.uiuc.edsg;

/**
 * Binary link that exhaustively iterates its children nodes
 */
public class CombinatoricLink extends BinaryLink {
	private boolean _isReset;
	
	public CombinatoricLink(IIterationLink left, IIterationLink right) {
		super(left, right);
		_isReset = true;
	}

	@Override
	public boolean hasNext() {
		return super.getLeft().hasNext() || super.getRight().hasNext();
	}

	@Override
	public void next() {
		IIterationLink left = super.getLeft();
		IIterationLink right = super.getRight();
		
		// Note that we iterate right first most often because
		//  the LinkCreator usually builds trees that are shallowest
		//  to the right.  This creates less recursion overhead.
		if (!right.hasNext()) {
			right.reset();
			if (!left.hasNext()) {
				throw new IterationException();
			}
			left.next();
		}
		right.next();
		if (_isReset) {
			left.next();
			_isReset = false;
		}
	}
	
	@Override
	public void reset() {
		super.reset();
		_isReset = true;
	}
	
	public void accept(IIteratorVisitor visitor) {
		visitor.visit(this);
	}

	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public BinaryLink createNewInstance(IIterationLink left, IIterationLink right) {
		return new CombinatoricLink(left, right);
	}
}
