package edu.uiuc.edsg;

/**
 * Callback for a generator that will be recursed on demand.
 * Used for recursive or circular data structures.
 */
public class Standin<ValueType> extends GeneratorBase<ValueType> {
	private Standinable<ValueType> standin;
	private IGenerator<ValueType> actual;
	
	public Standin(Standinable<ValueType> standin) {
		this.standin = standin;
	}
	
	protected IGenerator<ValueType> getActual() {
		if (actual == null) {
			actual = standin.expandStandin();
		}
		return actual;
	}

	@Override
	public boolean hasNext() {
		return getActual().hasNext();
	}
	
	@Override
	protected ValueType generateNext() {
		return getActual().next();
	}
	
	public void reset() {
		super.reset();
		if (actual != null) {
			actual.reset();
		}
	}	
}
