package edu.uiuc.edsg.samples;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import edu.uiuc.edsg.Chain;
import edu.uiuc.edsg.CompositeGenerator;
import edu.uiuc.edsg.IGenerator;
import edu.uiuc.edsg.IIterationLink;
import edu.uiuc.edsg.LinkCreator;

/**
 * A generator that demonstrates "higher-order" child generators.
 */
public class BuilderDemoGenerator extends CompositeGenerator<String> {
	
	IGenerator<String> baseGen; // supplied by user	
	Chain<NumberBuilder> numChain = new Chain<NumberBuilder>(new OneBuilder(), new TwoBuilder());
	Chain<LetterBuilder> letterChain = new Chain<LetterBuilder>(new ABuilder(), new BBuilder());
	
	public BuilderDemoGenerator(IGenerator<String> baseGen) {
		this.baseGen = baseGen;
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		return LinkCreator.createCombinatoricLink(baseGen, numChain, letterChain);
	}
	
	@Override
	protected String generateCurrent() {		
		String base = baseGen.current();
		NumberBuilder numBuilder = numChain.current();
		LetterBuilder letterBuilder = letterChain.current();
		
		// thread results through generated builders
		List<String> result1 = numBuilder.build(base);	
		List<String> result2 = letterBuilder.build(result1);		
		// process the result before returning it to the caller
		String processedResult = postProcess(result2);

		return processedResult;
	}

	private String postProcess(List<String> result2) {
		StringBuilder sb = new StringBuilder();
		for (String string : result2) {
			sb.append(string);
		}
		return sb.toString();
	}
	
	//// Builder implementations
	
	private static abstract class NumberBuilder {
		abstract List<String> build(String base);
	}
	
	private static class OneBuilder extends NumberBuilder {
		@Override
		List<String> build(String base) {
			return Arrays.asList(base, "1");
		}
	}
	
	private static class TwoBuilder extends NumberBuilder {
		@Override
		List<String> build(String base) {
			return Arrays.asList(base, "2");
		}
	}
	
	private static abstract class LetterBuilder {
		abstract List<String> build(List<String> bases);
	}
	
	private static class ABuilder extends LetterBuilder {
		@Override
		List<String> build(List<String> bases) {
			ArrayList<String> l = new ArrayList<String>();
			l.addAll(bases);
			l.add("A");
			return l;
		}
	}
	
	private static class BBuilder extends LetterBuilder {
		@Override
		List<String> build(List<String> bases) {
			ArrayList<String> l = new ArrayList<String>();
			l.addAll(bases);
			l.add("B");
			return l;
		}
	}

}
