package edu.uiuc.edsg.samples;

public class Graph<ValueType> {

	private java.util.Map<ValueType, GraphNode> _nodes;
	
	public Graph() {
		_nodes = new java.util.HashMap<ValueType, GraphNode>();
	}
	
	public void addNode(ValueType newNodeValue) {
		if (_nodes.containsKey(newNodeValue)) {
			throw new IllegalArgumentException(String.format("Cannot add duplicate key \"%1$d\"", newNodeValue));
		}
		GraphNode newNode = new GraphNode(newNodeValue);		
		_nodes.put(newNodeValue, newNode);
	}
	
	public void linkNodes(ValueType fromName, ValueType toName) {
		if (!_nodes.containsKey(fromName)) {
			throw new IllegalArgumentException(String.format("Cannot find node \"%1$d\"", fromName));
		}
		if (!_nodes.containsKey(toName)) {
			throw new IllegalArgumentException(String.format("Cannot find node \"%1$d\"", toName));
		}

		GraphNode fromNode = _nodes.get(fromName);
		GraphNode toNode = _nodes.get(toName);
		
		fromNode.addReference(toNode);
	}
	
	public int nodeCount() {
		return _nodes.size();
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		for (GraphNode node : _nodes.values()) {
			sb.append(node.toString());
		}
		return sb.toString();
	}
	
	public class GraphNode {
		java.util.Map<ValueType, GraphNode> _references;
		private ValueType _value;
		
		public GraphNode(ValueType value) {
			_references = new java.util.HashMap<ValueType, GraphNode>();
			_value = value;
		}
		
		public ValueType getValue() {
			return _value;
		}
		
		public void addReference(GraphNode otherNode) {
			_references.put(otherNode.getValue(), otherNode);
		}	
		
		public void clearReferences() {
			_references.clear();
		}
		
		public String toString() {
			StringBuilder sb = new StringBuilder();
			sb.append("\"").append(_value.toString()).append("\" references ");
			if (_references.size() == 0) {
				sb.append("nothing");
			}
			else {
				boolean isFirst = true;
				for (GraphNode reference : _references.values()) {
					if (!isFirst) {
						sb.append(", ");
					}
					sb.append("\"").append(reference.getValue().toString()).append("\"");
					isFirst = false;			
				}				
			}
			sb.append("\n");
			return sb.toString();
		}
			
	}
}
