package edu.uiuc.edsg.test;

import org.junit.Test;

import edu.uiuc.edsg.IGenerator;
import edu.uiuc.edsg.Chain;
import edu.uiuc.edsg.Wrapper;
import edu.uiuc.edsg.samples.Pair;
import edu.uiuc.edsg.samples.PairGenerator;


/**
 * Placeholder for Wrapper tests. Wrapper tests are usually implemented in derived class test fixtures.
 * 
 * @see SimpleWrapperTest
 * @see ConversionWrapperTest
 * 
 * @author Brett Daniel
 *
 */
public class WrapperTest {
	
	/**
	 * Basic demo of a custom wrapper that depends on two targets
	 */
	@Test()
	public void customMultiWrapper() {		
		final IGenerator<Integer> target1 = new Chain<Integer>(1, 2);
		final IGenerator<Character> target2 = new Chain<Character>('a', 'b', 'c');		
		IGenerator<String> wrapper = new Wrapper<String>(target1, target2) {			
			@Override
			protected String getDependentValue() {	
				// wrapper concatenates target values
				return target1.current().toString() + target2.current().toString();
			}
		};
		
		PairGenerator<Integer, Character> targetPair = 
			new PairGenerator<Integer, Character>(target1, target2);		
		PairGenerator<String, Pair<Integer, Character>> mainGen = 
			new PairGenerator<String, Pair<Integer,Character>>(wrapper, targetPair);
		
		GeneratorTestUtil.testBasicFunctionality(mainGen, 
				"[1a,[1,a]]",
				"[1b,[1,b]]",
				"[1c,[1,c]]",
				"[2a,[2,a]]",
				"[2b,[2,b]]",
				"[2c,[2,c]]");		
	}
	
}
