package edu.uiuc.astgen.basic;

import java.util.LinkedList;

import edu.uiuc.edsg.*;

import org.eclipse.jdt.core.dom.*;

import edu.uiuc.astgen.ASTNodeGeneratorBase;

public class BlockGenerator extends ASTNodeGeneratorBase<Block> {
	
	private java.util.LinkedList<ASTNodeGeneratorBase<? extends Statement>> _statementGenList;
		
	public BlockGenerator() {
		super();
		_statementGenList = new LinkedList<ASTNodeGeneratorBase<? extends Statement>>();
	}
	
	public BlockGenerator(ASTNodeGeneratorBase<? extends Statement>... statementGens) {
		this();
		for (ASTNodeGeneratorBase<? extends Statement> statementGen : statementGens) {
			_statementGenList.add(statementGen);
		}
	}	
	
	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator lc = new LinkCreator(LinkCreator.LinkType.COMBINATORIC);
		lc.addAll(_statementGenList);
		return lc.getLink();
	}

	public void addStatementGenerator(ASTNodeGeneratorBase<? extends Statement> newStatementGen) {
		_statementGenList.add(newStatementGen);
	}
	
	@Override
	protected Block generateCurrent() {	
		Block block = super.getAST().newBlock();
		for (ASTNodeGeneratorBase<? extends Statement> statementGen : _statementGenList) {
			block.statements().add(statementGen.current());	
		}
		
		return block;
	}
}
