package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.ConditionalExpression;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.PrimitiveType;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.edsg.IIterationLink;
import edu.uiuc.edsg.LinkCreator;

public class ConditionalExpressionGenerator extends ASTNodeGeneratorBase<ConditionalExpression>{
	private ASTNodeGeneratorBase<? extends Expression> conditionExpressionDeclaration;	
	private ASTNodeGeneratorBase<? extends Expression> thenExpressionDeclaration;
	private ASTNodeGeneratorBase<? extends Expression> elseExpressionDeclaration;
	
	public ConditionalExpressionGenerator(ASTNodeGeneratorBase<? extends Expression> conditionExpressionDeclaration, ASTNodeGeneratorBase<? extends Expression> thenExpressionDeclaration, ASTNodeGeneratorBase<? extends Expression> elseExpressionDeclaration) {
		super();
		this.conditionExpressionDeclaration = conditionExpressionDeclaration;
		this.thenExpressionDeclaration = thenExpressionDeclaration;
		this.elseExpressionDeclaration = elseExpressionDeclaration;
	}

	@Override
	protected ConditionalExpression generateCurrent() {
		ConditionalExpression conditionalExpression = super.getAST().newConditionalExpression();
		conditionalExpression.setThenExpression(thenExpressionDeclaration.current());
		conditionalExpression.setElseExpression(elseExpressionDeclaration.current());
		conditionalExpression.setExpression(conditionExpressionDeclaration.current());
		return conditionalExpression;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator lc = new LinkCreator(LinkCreator.LinkType.COMBINATORIC);
		lc.add(conditionExpressionDeclaration);
		lc.add(thenExpressionDeclaration);
		lc.add(elseExpressionDeclaration);
		return lc.getLink();
	}
	
	public static ConditionalExpressionGenerator getNewDefaultConditionalExpressionGenerator() {
		NumberLiteralGenerator numGen1 = new NumberLiteralGenerator(2, PrimitiveType.BYTE);
		NumberLiteralGenerator numGen2 = new NumberLiteralGenerator(2, PrimitiveType.BYTE);
		
		PrimitiveTypeGenerator primitiveTypeGen = new PrimitiveTypeGenerator(PrimitiveType.BOOLEAN, PrimitiveType.BYTE);		
		SimpleNameGenerator nameGen = new SimpleNameGenerator("v1");
		SingleVariableDeclarationGenerator variableGen = new SingleVariableDeclarationGenerator(nameGen, primitiveTypeGen);		
		InfixExpressionGenerator infixGenerator = new InfixExpressionGenerator(variableGen);		
		
		ConditionalExpressionGenerator conditionalExpGen = new ConditionalExpressionGenerator(infixGenerator, numGen1, numGen2);
		return conditionalExpGen;
	}

}
