package edu.uiuc.astgen.basic;

import java.util.LinkedList;

import org.eclipse.jdt.core.dom.Block;
import org.eclipse.jdt.core.dom.CatchClause;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ReturnStatement;
import org.eclipse.jdt.core.dom.TryStatement;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.edsg.IIterationLink;
import edu.uiuc.edsg.LinkCreator;

public class TryStatementGenerator extends ASTNodeGeneratorBase<TryStatement>{
	private LinkedList<ASTNodeGeneratorBase<CatchClause>> catchesDeclaration;
	private ASTNodeGeneratorBase<Block> bodyDeclaration;
	private ASTNodeGeneratorBase<Block> optionalFinallyDeclaration;
	
	public TryStatementGenerator(
			ASTNodeGeneratorBase<Block> bodyGen, 
			ASTNodeGeneratorBase<Block> optionaFinallyGen) {
		super();
		bodyDeclaration = bodyGen;
		optionalFinallyDeclaration = optionaFinallyGen;
	}
	
	public void addCatchExpression(ASTNodeGeneratorBase<CatchClause> newCatchGen) {
		catchesDeclaration.add(newCatchGen);
	}	
	
	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator lc = new LinkCreator(LinkCreator.LinkType.COMBINATORIC);
		lc.add(bodyDeclaration);
		lc.add(optionalFinallyDeclaration);
		lc.addAll(catchesDeclaration);
		return lc.getLink();
	}
	
	@Override
	protected TryStatement generateCurrent() {
		TryStatement tryStatement = super.getAST().newTryStatement();
		tryStatement.setBody(bodyDeclaration.current());
		tryStatement.setFinally(optionalFinallyDeclaration.current());
		for (ASTNodeGeneratorBase<CatchClause> catchGen : catchesDeclaration) {
			tryStatement.catchClauses().add(catchGen.current());	
		}			
		return tryStatement;
	}
}
