package edu.uiuc.astgen.nestedexpressions;

import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.Name;

import edu.uiuc.edsg.IGenerator;
import edu.uiuc.edsg.IIterationLink;
import edu.uiuc.edsg.LinkCreator;
import edu.uiuc.edsg.Chain;

public class ConditionalBuilderGenerator extends BuilderGenerator {
	
	private Chain<ConditionalOperandLocation> locationGen;
	
	public ConditionalBuilderGenerator(OperandType operandType) {
		super(operandType);
	}
	
	public void setLocations(ConditionalOperandLocation...locations) {
		locationGen = new Chain<ConditionalOperandLocation>(locations);
	}
	public void setLocationsAll() {
		setLocations(ConditionalOperandLocation.values());
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		if (locationGen == null) {
			setLocationsAll();
		}
		
		return LinkCreator.createCombinatoricLink(locationGen); 
	}
	
	@Override
	protected NestedExpressionBuilder generateCurrent() {		
		return new NestedExpressionBuilder() {
			public Expression build(Expression innerExpression, Name dummyVariable) {
				return locationGen.current().build(innerExpression, dummyVariable, getOperandType());
			}
			public boolean applicableTo(OperandType operandType) {
				return operandType == getOperandType();
			}
			@Override
			public OperandType outputType() {
				return getOperandType();
			}
		};
	}
}
