package edu.uiuc.astgen.nestedexpressions;

import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.Name;
import org.eclipse.jdt.core.dom.PostfixExpression;

import edu.uiuc.edsg.IGenerator;
import edu.uiuc.edsg.IIterationLink;
import edu.uiuc.edsg.LinkCreator;
import edu.uiuc.edsg.Chain;

public class PostfixBuilderGenerator extends BuilderGenerator {

	private IGenerator<PostfixExpression.Operator> opGen; // determined by operandType	
	
	public PostfixBuilderGenerator(OperandType operandType) {
		super(operandType);
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		if (opGen == null) {
			opGen = new Chain<PostfixExpression.Operator>(getOperandType().getPostfixOperators());
		}
		return LinkCreator.createCombinatoricLink(opGen);
	}
	@Override
	protected NestedExpressionBuilder generateCurrent() {			
		return new NestedExpressionBuilder() {
			public Expression build(Expression innerExpression, Name dummyVariable) {
				PostfixExpression postExpr = getDefaultAST().newPostfixExpression();
				postExpr.setOperator(opGen.current());
				postExpr.setOperand(innerExpression);
				return postExpr;
			}
			public boolean applicableTo(OperandType operandType) {
				return operandType == getOperandType();
			}
			@Override
			public boolean isAssignment() {
				return true;
			}
			@Override
			public OperandType outputType() {
				return getOperandType();
			}
		};
	}

}
