package edu.uiuc.astgen.nestedexpressions;

import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.Name;
import org.eclipse.jdt.core.dom.PrefixExpression;

import edu.uiuc.edsg.IGenerator;
import edu.uiuc.edsg.IIterationLink;
import edu.uiuc.edsg.LinkCreator;
import edu.uiuc.edsg.Chain;

public class PrefixBuilderGenerator extends BuilderGenerator {

	private IGenerator<PrefixExpression.Operator> opGen; // determined by operandType	
	
	public PrefixBuilderGenerator(OperandType operandType) {
		super(operandType);
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		if (opGen == null) {
			opGen = new Chain<PrefixExpression.Operator>(getOperandType().getPrefixOperators());
		}
		return LinkCreator.createCombinatoricLink(opGen);
	}
	@Override
	protected NestedExpressionBuilder generateCurrent() {			
		return new NestedExpressionBuilder() {
			public Expression build(Expression innerExpression, Name dummyVariable) {
				PrefixExpression preExpr = getDefaultAST().newPrefixExpression();
				preExpr.setOperator(opGen.current());
				preExpr.setOperand(innerExpression);
				return preExpr;
			}
			public boolean applicableTo(OperandType operandType) {
				return operandType == getOperandType();
			}
			@Override
			public boolean isAssignment() {
				return true;
			}
			@Override
			public OperandType outputType() {
				return getOperandType();
			}
		};
	}

}
